/****************************************************************************
** $Id: quickextensions.cpp  beta1   edited Dec 13 10:09 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickextensions.h"

#include "quickqtwidgets.h"
#include "quicksystem.h"
#include "quickqtdialogs.h"
#include "quickqtkernel.h"
#include "quickqtnamespace.h"
#if defined(QT_MODULE_XML) && !defined(QSA_NO_XML)
#include "quickqtdom.h"
#endif
#if defined(QT_MODULE_TABLE) && !defined(QSA_NO_TABLE)
#include "quickqtable.h"
#endif
#if defined(QT_MODULE_SQL) && !defined(QSA_NO_SQL)
#include "quickqtsql.h"
#endif
#if defined(QT_MODULE_NETWORK) && !defined(QSA_NO_NETWORK)
#include "quickqtnetwork.h"
#endif

#include <qptrlist.h>
#include <qsocket.h>
#include <qwidgetfactory.h>
#include <qlayout.h>
#include <qvariant.h>

#define ARGS( x ) ( args.size() > x ? args[x] : QVariant() )

QuickExtensions::QuickExtensions( QuickInterpreter *i )
    : QuickDispatchObjectFactory( i )
{
}

QStringList QuickExtensions::featureList() const
{
    static QStringList *features = 0;
    if ( features )
	return *features;
    QStringList lst;

    // widgets
    lst << "QWidget" << "QDateEdit" << "QListViewItem" << "QListView"
	<< "ToolTip" << "WhatsThis" << "WidgetFactory"
	<< "QPopupMenu" << "QWidgetStack" << "QHBoxLayout" << "QVBoxLayout"
	<< "QListBox" << "QListBoxItem" << "QComboBox" << "QTextEdit";

#if defined(QT_MODULE_TABLE) && !defined(QSA_NO_TABLE)
    lst << "QTable";
#endif
#if defined(QT_MODULE_SQL) && !defined(QSA_NO_SQL)
    // sql
    lst << "Database" << "QSqlRecord" << "QSqlField" << "QSqlQuery" << "QSqlCursor" << "QSqlIndex";
    lst << "QDataTable";
#endif

#if defined(QT_MODULE_NETWORK) && !defined(QSA_NO_NETWORK)
    // network
    lst << "Network" << "QSocket";
#endif

    // system
    lst << "System";

#if defined(QT_MODULE_XML) && !defined(QSA_NO_XML)
    // dom
    lst << "DomNode" << "DomElement" << "DomText" << "DomDocument";
    lst << "QDomNode" << "QDomElement" << "QDomText" << "QDomDocument";
#endif

    // dialogs
    lst << "FileDialog" << "MessageBox" << "InputDialog" << "ColorDialog";

    // kernel
    lst << "QProcess" << "QFontDatabase" << "FontDatabase" << "File" << "QFile" << "Dir" << "QDir";

    // namespace
    lst << "Qt" << "QuickNamespace";

    // other
    lst << "QObjectList";

    QStringList widgets = QWidgetFactory::widgets();
    for ( QStringList::Iterator it = widgets.begin(); it != widgets.end(); ++it ) {
	if ( lst.find( *it ) == lst.end() )
	    lst << *it;
    }

    features = new QStringList( lst );
    return *features;
}

bool QuickExtensions::createInterface( const QCString &className, void *ptr )
{
    if ( className == "QWidget" ) {
	addInterface( new QuickWidgetInterface( (QWidget*)ptr ) );
    } else if ( className == "ToolTip" ) {
	addInterface( new QuickToolTip );
    } else if ( className == "WhatsThis" ) {
	addInterface( new QuickWhatsThis );
    } else if ( className == "QDateEdit" ) {
	addInterface( new QuickDateEditInterface( (QDateEdit*)ptr ) );
#if defined(QT_MODULE_TABLE) && !defined(QSA_NO_TABLE)
    } else if ( className == "QTable" ) {
	addInterface( new QuickTableInterface( (QTable*)ptr ) );
#endif
    } else if ( className == "QListView" ) {
	addInterface( new QuickListViewInterface( (QListView*)ptr ) );
    } else if ( className == "QListBox" ) {
	addInterface( new QuickListBoxInterface( (QListBox*)ptr ) );
    } else if ( className == "QTextEdit" ) {
	addInterface( new QuickTextEditInterface( (QTextEdit*)ptr ) );
    } else if ( className == "QComboBox" ) {
	addInterface( new QuickComboBoxInterface( (QComboBox*)ptr ) );
    } else if ( className == "QPopupMenu" ) {
	addInterface( new QuickPopupMenuInterface( (QPopupMenu*)ptr ) );
    } else if ( className == "QWidgetStack" ) {
	addInterface( new QuickWidgetStackInterface( (QWidgetStack*)ptr ) );
    } else if ( className == "QHBoxLayout" ) {
	addInterface( new QuickHBoxLayoutInterface( (QHBoxLayout*)ptr ) );
    } else if ( className == "QVBoxLayout" ) {
        addInterface( new QuickVBoxLayoutInterface( (QVBoxLayout*)ptr ) );
    } else if ( className == "QListViewItem" ) {
	addInterface( new QListViewItemPtr( (QListViewItem*)ptr ) );
    } else if ( className == "QListBoxItem" ) {
	addInterface( new QListBoxItemPtr( (QListBoxItem*)ptr ) );
    } else if ( className == "WidgetFactory" ) {
	addInterface( new QuickWidgetFactory );
#if defined(QT_MODULE_SQL) && !defined(QSA_NO_SQL)
    } else if ( className == "Database" ) {
	addInterface( new QuickDatabase );
    } else if ( className == "QSqlQuery" ) {
	addInterface( new QuickDatabaseQueryPtr( (QSqlQuery*)ptr ) );
    } else if ( className == "QSqlField" ) {
	addInterface( new QuickDatabaseFieldPtr( (QSqlField*)ptr ) );
    } else if ( className == "QSqlRecord" ) {
	addInterface( new QuickDatabaseRecordPtr( (QSqlRecord*)ptr ) );
    } else if ( className == "QSqlCursor" ) {
	addInterface( new QuickDatabaseCursorPtr( (QSqlCursor*)ptr ) );
    } else if ( className == "QSqlIndex" ) {
	addInterface( new QuickDatabaseIndexPtr( (QSqlIndex*)ptr ) );
    } else if ( className == "QDataTable" ) {
	addInterface( new QuickDataTableInterface( (QDataTable*)ptr ) );
#endif
#if defined(QT_MODULE_NETWORK) && !defined(QSA_NO_NETWORK)
    } else if ( className == "Network" ) {
	addInterface( new QuickNetwork );
    } else if ( className == "QSocket" ) {
// 	result.resize( 2 );
// 	result.insert( 0, (QSocket*)ptr );
// 	result.insert( 1, new QuickSocket( (QSocket*)ptr ) );
	addInterface( new QuickSocket( (QSocket*)ptr ) );
#endif
    } else if ( className == "System" ) {
	addInterface( new System );
#if defined(QT_MODULE_XML) && !defined(QSA_NO_XML)
    } else if ( className == "QDomNode" ) {
	addInterface( new DomNode( (QDomNode*)ptr ) );
    } else if ( className == "QDomElement" ) {
	addInterface( new DomElement( (QDomElement*)ptr ) );
    } else if ( className == "QDomText" ) {
	addInterface( new DomText( (QDomText*)ptr ) );
    } else if ( className == "QDomDocument" ) {
	addInterface( new DomDocument( (QDomDocument*)ptr ) );
#endif
    } else if ( className == "FileDialog" ) {
	addInterface( new QuickFileDialogInterface );
    } else if ( className == "MessageBox" ) {
	addInterface( new QuickMessageBoxInterface );
    } else if ( className == "ColorDialog" ) {
	addInterface( new QuickColorDialogInterface );
    } else if ( className == "InputDialog" ) {
	addInterface( new QuickInputDialogInterface );
    } else if ( className == "Qt" || className == "QuickNamespace" ) {
	addInterface( new QuickNamespace );
    } else if ( className == "QObjectList" ) {
	addInterface( new QObjectListPtr( (QObjectList*)ptr ) );
    } else {
	return FALSE;
    }

    return TRUE;
}

bool QuickExtensions::isGlobal( const QCString &className ) const
{
    if ( className == "Database" || className == "MessageBox" ||
	 className == "Network" || className == "ToolTip" || className == "WhatsThis" ||
	 className == "WidgetFactory" || className == "System" ||
	 className == "FileDialog" || className == "ColorDialog" || className == "InputDialog" ||
	 className == "Qt" || className == "QuickNamespace" )
	return TRUE;
    return FALSE;
}

bool QuickExtensions::isConstructor( const QString &className ) const
{
    if ( /*className == "DomNode" ||
	 className == "DomElement" ||
	 className == "DomText" || */
	 className == "DomDocument" )
	return TRUE;
    if ( className == "QListViewItem" )
	return TRUE;
    if ( className == "QObjectList" )
	return TRUE;
    if ( className == "QProcess" )
	return TRUE;
    if ( className == "QFontDatabase" || className == "FontDatabase" )
	return TRUE;
    if ( className == "QFile" || className == "File" )
	return TRUE;
    if ( className == "QDir" || className == "Dir" )
	return TRUE;
    if ( className == "QVBoxLayout" || className == "QHBoxLayout" )
	return TRUE;
    if ( QWidgetFactory::supportsWidget( className ) )
	return TRUE;

    return FALSE;
}

bool QuickExtensions::construct( const QString & className,
				 const QValueList<QVariant> &args )
{
#if defined(QT_MODULE_XML) && !defined(QSA_NO_XML)
    if ( className == "DomDocument" ) {
	addInterface( new DomDocument( ARGS(0).toString() ) );
    } else
#endif
    if ( className == "QListViewItem" ) {
	if ( args.count() == 1 ) {
	    ObjAttr oa = extractObject( args[0] );
	    if ( oa.num ) {
		if( !oa.isQObject ) {
		    QListViewItem *i = (QListViewItem*)oa.num;
		    if ( i ) {
		        addInterface( new QListViewItemPtr( new QListViewItem( i ) ) );
			return TRUE;
		    }
		}
		if ( ( (QObject*)oa.num )->inherits( "QListView" ) ) {
		    QListView *lv = (QListView*)oa.num;
		    addInterface( new QListViewItemPtr( new QListViewItem( lv ) ) );
		    return TRUE;
		}
	    }
	}
    } else if ( className == "QProcess" ) {
	QString s = ARGS(0).toString();
	if ( s.find( ' ' ) == -1 ) {
	    addInterface( new QuickProcess( s ) );
	} else {
	    QStringList lst = QStringList::split( ' ', s );
	    addInterface( new QuickProcess( lst ) );
	}
    } else if ( className == "QFontDatabase" || className == "FontDatabase" ) {
	addInterface( new QuickFontDatabase );
    } else if ( className == "QFile" || className == "File" ) {
	QString s = ARGS(0).toString();
	addInterface( new QuickFile( s ) );
    } else if ( className == "QDir" || className == "Dir" ) {
	QString s = ARGS(0).toString();
	addInterface( new QuickDir( s ) );
    } else if ( QWidgetFactory::supportsWidget( className ) ||
		className == "QVBoxLayout" || className == "QHBoxLayout" ) {
	QObject *parent = 0;
	QWidget *parentWidget = 0;
	const char *name = 0;

	ObjAttr oa = extractObject( ARGS(0) );
	if ( oa.num && oa.isQObject ) {
	    parent = (QObject*)oa.num;
	    if ( parent->isWidgetType() )
	        parentWidget = (QWidget*)parent;
	}
	if ( ARGS(1).type() == QVariant::String || ARGS(1).type() == QVariant::CString )
	    name = qstrdup( ARGS(1).toString().latin1() );

	QObject *w = 0;
	if ( className == "QVBoxLayout" ) {
	    if ( parentWidget )
		w = new QVBoxLayout( parentWidget, 6, 6, name );
	    else
		w = new QVBoxLayout( (QLayout*)parent, 6, name );
	    addObject( w );
	    addInterface( new QuickVBoxLayoutInterface( (QVBoxLayout*)w ) );
	} else if ( className == "QHBoxLayout" ) {
	    if ( parentWidget )
		w = new QHBoxLayout( parentWidget, 6, 6, name );
	    else
		w = new QHBoxLayout( (QLayout*)parent, 6, name );
	    addObject( w );
	    addInterface( new QuickHBoxLayoutInterface( (QHBoxLayout*)w ) );
	} else {
	    QWidgetFactory f;
	    w = f.createWidget( className, parentWidget, name );
	    addObject( w );
	}
    } else {
	return FALSE;
    }

    return TRUE;
}

ObjAttr QuickExtensions::extractObject( const QVariant &arg ) const
{
    ObjAttr oa( 0, FALSE );
    if ( arg.type() == QVariant::String || arg.type() == QVariant::CString ) {
	static QString pointer_header = "Pointer";
	QString s = arg.toString();
	if ( s.startsWith( pointer_header ) ) {
	    QStringList l = QStringList::split( ':', s );
	    if ( l.count() == 3 ) {
		oa.num = l[1].toULong();
		if ( l[2] == "QObject" )
		    oa.isQObject = TRUE;
		return oa;
	    }
	}
    }
    return oa;
}
