/****************************************************************************
** $Id: quickqtdialogs.h  beta2   edited Dec 10 13:07 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKQTDIALOGS_H
#define QUICKQTDIALOGS_H

#include "quickdispatchobject.h"
#include <qstringlist.h>

class QuickFileDialogInterface : public QuickUnnamedObject
{
    Q_OBJECT

public:
    QuickFileDialogInterface();
    ~QuickFileDialogInterface();

public slots:
    QString getOpenFileName( const QString &initially,
			     const QString &filter,
			     const QString &caption );
    QString getSaveFileName( const QString &initially,
			     const QString &filter,
			     const QString &caption );
    QString getExistingDirectory( const QString &dir,
				  const QString &caption,
				  bool showDirOnly );
    QStringList getOpenFileNames( const QString &filter,
				  const QString &dir,
				  const QString &caption );
    QString getOpenFileName() { return getOpenFileName( QString::null, QString::null, QString::null ); }
    QString getSaveFileName() { return getSaveFileName( QString::null, QString::null, QString::null ); }
    QString getExistingDirectory() { return getExistingDirectory( QString::null, QString::null, TRUE ); }
    QStringList getOpenFileNames() { return getOpenFileNames( QString::null, QString::null, QString::null ); }

};

class QuickColorDialogInterface : public QuickUnnamedObject
{
    Q_OBJECT

public:
    QuickColorDialogInterface();
    ~QuickColorDialogInterface();

public slots:
    QString getColor( const QString &initColor );
    QString getColor() { return getColor( QString::null ); }

};

class QuickInputDialogInterface : public QuickUnnamedObject
{
    Q_OBJECT

public:
    QuickInputDialogInterface();
    ~QuickInputDialogInterface();

public slots:
    QString getText( const QString &init, const QString &caption, const QString &label );
    int getInteger( int init, int from, int to, int step, const QString &caption, const QString &label );
    double getDouble( double init, double from, double to, int decimals,
		      const QString &caption, const QString &label );
    QString getItem( const QStringList &list, int current, bool editable,
		     const QString &caption, const QString &label );
    QString getText() { return getText( QString::null, QString::null, QString::null ); }
    int getInteger() { return getInteger( 0, -2147483647, 2147483647, 1, QString::null, QString::null ); }
    double getDouble() { return getDouble( 0.0, 2147483647, 2147483647, 1, QString::null, QString::null ); }
    QString getItem( const QStringList &list ) { return getItem( list, 0, TRUE, QString::null, QString::null ); }

};

class QuickMessageBoxInterface : public QuickUnnamedObject
{
    Q_OBJECT

public:
    QuickMessageBoxInterface();

public slots:
    void information( const QString &text, const QString &title );
    void warning( const QString &text, const QString &title );
    void critical( const QString &text, const QString &title );
    bool confirm( const QString &text, const QString &title );

};

#endif
