/****************************************************************************
** $Id: quickqtkernel.h  beta2   edited Dec 10 13:07 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKQTKERNEL_H
#define QUICKQTKERNEL_H

#include "quickdispatchobject.h"
#include <qprocess.h>
#include <qfontdatabase.h>
#include <qfile.h>
#include <qdir.h>

class QuickProcess : public QProcess
{
    Q_OBJECT

public:
    QuickProcess( const QString &arg0 );
    QuickProcess( const QStringList &args );

public slots:
    void start();
    void setArguments( const QStringList &args );
    void clearArguments();
    void addArgument( const QString &arg );
    QByteArray readStdout();
    QByteArray readStderr();
    bool canReadLineStdout() const;
    bool canReadLineStderr() const;
    QString readLineStdout();
    QString readLineStderr();

};

class QuickFontDatabase : public QuickUnnamedObject,
			  public QFontDatabase
{
    Q_OBJECT

public:
    QuickFontDatabase();

public slots:
    int numFamilies() const;
    QString family( int index ) const;

};

class QuickFile : public QuickUnnamedObject
{
    Q_OBJECT
    Q_PROPERTY( QString fileName READ fileName )
    Q_ENUMS( OpenMode )

public:
    enum OpenMode { ReadOnly = IO_ReadOnly,
		    WriteOnly = IO_WriteOnly,
		    ReadWrite = IO_ReadWrite,
		    Append = IO_Append,
		    Truncate = IO_Truncate,
		    Translate = IO_Translate
    };

    QuickFile( const QString &name );

    QString fileName() const;

public slots:
    bool open( int openMode );
    void close();
    QString readAll();
    QByteArray readAllRaw();
    QString readLine( bool removeEndingWhiteChars = TRUE );
    void writeBlock( const QString &data );
    void writeBlock( const QByteArray &data );
    void reset();
    void remove();
    bool exists();
    bool atEnd();

private:
    QFile file;

};


class QuickDir : public QuickUnnamedObject
{
    Q_OBJECT
    Q_PROPERTY( QString path READ path WRITE setPath )
    Q_PROPERTY( QString nameFilter READ nameFilter WRITE setNameFilter )
    Q_PROPERTY( int count READ count )
    Q_PROPERTY( bool readable READ isReadable )

public:
    QuickDir( const QString &path );

    QString path() const;
    QString nameFilter() const;
    int count() const;

    bool isReadable() const;

public slots:
    void setPath( const QString &path );
    void cd( const QString &relPath );
    void setNameFilter( const QString &filter );
    QString entry( int num ) const;

    void mkdir( const QString &dirName );
    void rmdir( const QString &dirName );
    void remove( const QString &fileName );
    void rename( const QString &oldName, const QString &newName );

private:
    QDir dir;

};

#endif
