/****************************************************************************
** $Id: quickqtsql.h  beta2   edited Dec 10 13:07 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKQTSQL_H
#define QUICKQTSQL_H

#include "quickdispatchobject.h"
#include <qvariant.h>
#include <qsqlrecord.h>

class QSqlDatabase;
class QSqlQuery;
class QSqlField;
class QSqlIndex;
class QSqlCursor;
class QDataTable;

class QuickDatabase : public QuickUnnamedObject
{
    Q_OBJECT

public:
    QuickDatabase();

public slots:
    bool addConnection( const QString& connectionname,
			const QString& databasename,
			const QString& driver,
			const QString& username,
			const QString& password,
			const QString& hostname,
			int port );

    bool addConnection( const QString& databasename,
			const QString& driver,
			const QString& username,
			const QString& password,
			const QString& hostname,
			int port );

    bool useDefaultConnection();

    bool useConnection( const QString& name );

    QStringList drivers() const;
    QStringList tables() const;
    QStringList tables( const QString &database ) const;
    QSqlQuery* createQuery() const;
    QSqlQuery* createQuery( const QString &query ) const;
    QSqlCursor* createCursor( const QString &name ) const;
    QSqlQuery* createQuery( const QString &query, const QString &database ) const;
    QSqlCursor* createCursor( const QString &name, const QString &database ) const;
    QSqlRecord record( const QString& tablename, const QString &database ) const;
    QSqlRecord record( const QString& tablename ) const;

//    void removeConnection();
//    void removeConnection( const QString& database );

private:
    QSqlDatabase* currentConnection( const QString& database = QString::null ) const;

    QString currentConnectionName;
    QString lastErrorText;

};

class QuickDatabaseQueryPtr : public QuickPtrDispatchObject
{
    Q_OBJECT

public:
    QuickDatabaseQueryPtr( QSqlQuery* query );
    ~QuickDatabaseQueryPtr();

public slots:
    bool isValid() const;
    bool isActive() const;
    bool isNull( int fieldNumber ) const;
    int at() const;
    int numRowsAffected() const;
    bool isSelect() const;
    int size() const;
    bool exec( const QString& query );
    QVariant value( int i ) const;
    bool seek( int recordNumber );
    bool next();
    bool prev();
    bool first();
    bool last();

private:
    QSqlQuery* query() const;

};

class QuickDatabaseFieldPtr : public QuickPtrDispatchObject
{
    Q_OBJECT

public:
    QuickDatabaseFieldPtr( QSqlField* field );
    ~QuickDatabaseFieldPtr();

public slots:
    QVariant value() const;
    QString name() const;
    bool isNull() const;
    QVariant::Type type() const; //## harri: is this return type valid?

private:
    QSqlField* field() const;

};

class QuickDatabaseRecordPtr : public QuickPtrDispatchObject
{
    Q_OBJECT

public:
    QuickDatabaseRecordPtr( QSqlRecord* query );
    ~QuickDatabaseRecordPtr();

public slots:
    void setValue( const QString& field, const QVariant& value );
    QVariant value( const QString& field ) const;

    QString fieldName( int fieldNumber ) const;
    int position( const QString& field ) const;
    QSqlField* field( const QString& field );
    bool contains( const QString& field ) const;
    void clearValues();
    void nullifyValues();
    uint count() const;

private:
    QSqlRecord* record() const;

};

class QuickDatabaseIndexPtr : public QuickPtrDispatchObject
{
    Q_OBJECT

public:
    QuickDatabaseIndexPtr( QSqlIndex* index );
    ~QuickDatabaseIndexPtr();

public slots:
    QVariant value( int fieldNumber ) const;
    uint count() const;
    QString fieldName( int i ) const;
    int position( const QString& name ) const;
    bool isDescending( int i );

private:
    QSqlIndex* index() const;

};

class QuickDatabaseCursorPtr : public QuickPtrDispatchObject
{
    Q_OBJECT

public:
    QuickDatabaseCursorPtr( QSqlCursor* cursor );
    ~QuickDatabaseCursorPtr();

public slots:
    QVariant value( const QString& field ) const;
    uint count() const;
    QString fieldName( int i ) const;
    int position( const QString& field ) const;
    QSqlField* field( const QString& field );
    bool contains( const QString& field ) const;

    void setSort( const QString& sort );
    QSqlIndex* sort() const;
    void setFilter( const QString& filter );
    QString filter() const;

    bool selectRecords();
    bool seek( int recordNumber );
    bool next();
    bool prev();
    bool first();
    bool last();

    QSqlRecord* primeInsert();
    QSqlRecord* primeUpdate();
    QSqlRecord* primeDelete();

    bool insertRecord();
    bool updateRecord();
    bool deleteRecord();

    QSqlRecord* editBuffer();
    QSqlIndex* primaryIndex();

private:
    class Private;
    Private* d;
    QSqlCursor* cursor() const;

};

class QuickDataTableInterface : public QObject
{
    Q_OBJECT

public:
    QuickDataTableInterface( QDataTable *t );

public slots:
    QSqlRecord *currentRecord() const;
    QSqlCursor* currentCursor() const;
    QSqlCursor* sqlCursor() const;
    bool find( QSqlIndex* index );
    void setCursor( QSqlCursor* cursor, bool autoPopulate ) { setSqlCursor( cursor, autoPopulate ); }
    void setSqlCursor( QSqlCursor* cursor, bool autoPopulate );
private:
    QDataTable *table;

};

#endif
