/****************************************************************************
** $Id: quickclassbrowserinterfaceimpl.cpp  beta2   edited Dec 10 13:07 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickclassbrowserinterfaceimpl.h"
#include "quickclassbrowser.h"

QuickClassBrowserInterfaceImpl::QuickClassBrowserInterfaceImpl()
    : ref( 0 )
{
    classBrowser = 0;
}

QuickClassBrowserInterfaceImpl::~QuickClassBrowserInterfaceImpl()
{
    delete classBrowser;
}

QRESULT QuickClassBrowserInterfaceImpl::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)(QComponentInformationInterface*)this;
    else if ( uuid == IID_QComponentInformation )
	*iface = (QComponentInformationInterface*)this;
    else if ( uuid == IID_ClassBrowser )
	*iface = (ClassBrowserInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}

QListView *QuickClassBrowserInterfaceImpl::createClassBrowser( QWidget *parent ) const
{
    if ( !classBrowser ) {
	( (QuickClassBrowserInterfaceImpl*)this )->classBrowser = new QuickClassBrowser( parent );
	connect( classBrowser, SIGNAL( clicked( QListViewItem * ) ),
		 this, SLOT( itemClicked( QListViewItem* ) ) );
    }
    return classBrowser;
}

void QuickClassBrowserInterfaceImpl::update( const QString &code ) const
{
    if ( classBrowser )
	classBrowser->update( code );
}

void QuickClassBrowserInterfaceImpl::clear() const
{
    if ( classBrowser )
	classBrowser->clear();
}

void QuickClassBrowserInterfaceImpl::onClick( QObject *receiver, const char *slot )
{
    connect( this, SIGNAL( jumpTo( const QString &, const QString &, int ) ),
	     receiver, slot );
}

void QuickClassBrowserInterfaceImpl::itemClicked( QListViewItem *i )
{
    if ( !i )
	return;
    switch ( i->rtti() ) {
    case QuickClassBrowserItem::Class:
	emit jumpTo( i->text( 0 ).left( i->text( 0 ).find( '[' ) ), QString::null, Class );
	break;
    case QuickClassBrowserItem::Function:
	emit jumpTo( i->text( 0 ).left( i->text( 0 ).find( '(' ) ) ,
		     ( i->parent() == i->listView()->firstChild() ? QString::null :
		       i->parent()->text( 0 ).left( i->parent()->text( 0 ).find( '[' ) ) ),
		     Function );
	break;
    default:
	break;
    }
}
