/****************************************************************************
** $Id: quickdebuggerfrontend.h  beta2   edited Jan 29 10:46 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKDEBUGGERFRONTEND_H
#define QUICKDEBUGGERFRONTEND_H

#include <actioninterface.h>
#include <qobject.h>

struct DesignerInterface;
class QObjectList;
class QWidget;
class QuickVariableView;
class QuickStackView;
class QMainWindow;
class EventKiller;
class QuickInterpreterInterfaceImpl;
class QuickEditorInterfaceImpl;
class QuickProvider;
class QActionGroup;
class QSProject;
class QSInterpreter;

class QuickDebuggerFrontend : public QObject, public ActionInterface
{
    Q_OBJECT

public:
    QuickDebuggerFrontend( QuickEditorInterfaceImpl *i );
    virtual ~QuickDebuggerFrontend();

    QRESULT queryInterface( const QUuid&, QUnknownInterface **iface );
    ulong addRef();
    ulong release();

    QStringList featureList() const;
    QAction* create( const QString&, QObject* parent = 0 );
    QString group( const QString & ) const;
    void connectTo( QUnknownInterface *appInterface );
    bool location( const QString &name, Location l ) const;

    void runFunction( const QString &function );

private slots:
    void runProject();
    void runFunction();
    void startProject();
    void stopProject();
    void projectChanged();
    void debuggerStopped( bool &ret );
    void debugNext();
    void debugStep();
    void debugContinue();
    void runtimeError();
    void objectDestroyed( QObject *o );

    void toggleDebugger( bool enable );

private:
    void enterSession();
    void leaveSession();
    void setupWatchView();
    void setupCallStack();
    void getMainWindow();
    void init();

private:
    ulong ref;
    DesignerInterface *dIface;
    QAction *actionRun, *actionRunFunction;
    QActionGroup *groupRun;
    QAction *actionStop;
    QAction *actionNext;
    QAction *actionStep;
    QObjectList *runningProject;
    bool modal;
    QuickVariableView *watchVars;
    QuickStackView *callStack;
    QMainWindow *mw;
    EventKiller *eventKiller;
    QuickInterpreterInterfaceImpl *interpreterInterface;
    QuickEditorInterfaceImpl *editorIface;
    QSProject *project;
    QSInterpreter *interp;
    bool executeProjectEntryPoint;
    bool haveGUI;
    bool debuggerEnabled;

};

#endif
