/****************************************************************************
** $Id: quickprograminterfaceimpl.cpp  beta2   edited Jan 22 18:39 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickprograminterfaceimpl.h"
#include "quickinterpreter.h"
#include "../qsa/qsproject.h"

QuickProgramInterfaceImpl::QuickProgramInterfaceImpl()
    : ProgramInterface(), ref( 0 )
{
}

QRESULT QuickProgramInterfaceImpl::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)this;
    else if ( uuid == IID_Program )
	*iface = (ProgramInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}

ulong QuickProgramInterfaceImpl::addRef()
{
    return ref++;
}

ulong QuickProgramInterfaceImpl::release()
{
    if ( !--ref ) {
	delete this;
	return 0;
    }
    return ref;
}

bool QuickProgramInterfaceImpl::check( const QString &code, QStringList &errors, QValueList<uint> &lines )
{
    if( !QSProject::isIDEOpen() )
	return FALSE;

    QuickInterpreter *engine = QSProject::ideQuickInterpreter();
    engine->checkSyntax( code );
    if ( engine->hadError() ) {
	errors.clear();
	lines.clear();
	errors += engine->errorMessages();
	lines += engine->errorLines();
	return FALSE;
    }
    return TRUE;
}

bool QuickProgramInterfaceImpl::build( const QString &, QMap< QString, QMap<QStringList, int > > & )
{
    return FALSE;
}

int QuickProgramInterfaceImpl::run( const QStringList & )
{
    return FALSE;
}
