/****************************************************************************
** $Id: quickprojectsettings.ui.h  beta2   edited Jan 10 13:39 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickinterpreter.h"
#include "qsinterpreter.h"

void QuickProjectSettings::init()
{
}

void QuickProjectSettings::destroy()
{

}

void QuickProjectSettings::reInit( QUnknownInterface * iface )
{
    listForms->clear();
    DesignerInterface *dIface = 0;
    iface->queryInterface( IID_Designer, (QUnknownInterface**)&dIface );
    if ( !dIface )
	return;
    QStringList l = dIface->currentProject()->formNames();
    QString form = dIface->currentProject()->customSetting( "MAINFORM" );
    QString type = dIface->currentProject()->customSetting( "QUICK_APPTYPE" );
    if ( type.isEmpty() )
	type = "GUI";
    QString useServer = dIface->currentProject()->customSetting( "USE_SERVER" );
    QString server = dIface->currentProject()->customSetting( "QUICK_HOST" );
    QString user = dIface->currentProject()->customSetting( "QUICK_USER" );
    QString passwd = dIface->currentProject()->customSetting( "QUICK_PASSWD" );
    QString method = dIface->currentProject()->customSetting( "QUICK_CGI_METHOD" );
    QString browser = dIface->currentProject()->customSetting( "QUICK_CGI_BROWSER_COMMAND" );
    QString query = dIface->currentProject()->customSetting( "QUICK_CGI_QUERY" );
    QString args = dIface->currentProject()->customSetting( "QUICK_APP_ARGS" );
    QString packages = dIface->currentProject()->customSetting( "QUICK_PACKAGES" );
    if ( browser.isEmpty() )
	browser = "netscape %s";

    QStringList::Iterator it;
    if ( !dIface->singleProjectMode() ) {
	for ( it = l.begin(); it != l.end(); ++it ) {
	    listForms->insertItem( *it );
	    if ( *it == form )
		listForms->setCurrentItem( listForms->count() - 1 );
	}
    }

    comboAppType->setCurrentItem( 0 );
    for ( int i = 0; i < comboAppType->count(); ++i ) {
	if ( comboAppType->text( i ) == type ) {
	    comboAppType->setCurrentItem( i );
	    listForms->setEnabled( i == 0 );
	    break;
	}
    }

    if ( method == "GET" )
	comboMethod->setCurrentItem( 0 );
    else
	comboMethod->setCurrentItem( 1 );
    setupQueryList( query );
    editBrowser->setText( browser );

    checkServer->setChecked( useServer.lower() == "yes" );
    groupServer->setEnabled( useServer.lower() == "yes" );
    editServer->setText( server );
    editUser->setText( user );
    editPasswd->setText( passwd );
    editArgs->setText( args );
    editPackages->setText( packages );

    appTypeChanged( type );

    if ( dIface->singleProjectMode() ) {
	groupServer->hide();
	checkServer->hide();
	line->hide();
	labelArgs->hide();
	editArgs->hide();
	tabConfig->setTabEnabled( tabCGI, FALSE );
	tabConfig->setTabEnabled( tabGeneral, FALSE );
	comboAppType->setEnabled( FALSE );
	labelMain->setText( "Choose the function which should be called on run:" );
	QStringList l = QSInterpreter::defaultInterpreter()->globalFunctions();
	for ( QStringList::ConstIterator it = l.begin(); it != l.end(); ++it ) {
	    listForms->insertItem( *it + "()" );
	    if ( QString( *it + "()" ) == form )
		listForms->setCurrentItem( listForms->count() - 1 );
	}
    }
}

void QuickProjectSettings::save( QUnknownInterface * iface )
{
    DesignerInterface *dIface = 0;
    iface->queryInterface( IID_Designer, (QUnknownInterface**)&dIface );
    if ( !dIface )
	return;
    dIface->currentProject()->setCustomSetting( "MAINFORM", listForms->text( listForms->currentItem() ) );
    dIface->currentProject()->setCustomSetting( "QUICK_APPTYPE", comboAppType->currentText() );
    dIface->currentProject()->setCustomSetting( "USE_SERVER", checkServer->isChecked() ? "yes" : "no" );
    dIface->currentProject()->setCustomSetting( "QUICK_HOST", editServer->text() );
    dIface->currentProject()->setCustomSetting( "QUICK_USER", editUser->text() );
    // ##### don't save password in clear text!!
    dIface->currentProject()->setCustomSetting( "QUICK_PASSWD", editPasswd->text() );
    dIface->currentProject()->setCustomSetting( "QUICK_CGI_METHOD", comboMethod->currentText() );
    dIface->currentProject()->setCustomSetting( "QUICK_CGI_QUERY", composeQuery() );
    dIface->currentProject()->setCustomSetting( "QUICK_CGI_BROWSER_COMMAND", editBrowser->text() );
    dIface->currentProject()->setCustomSetting( "QUICK_APP_ARGS", editArgs->text() );
    dIface->currentProject()->setCustomSetting( "QUICK_PACKAGES", editPackages->text() );
}

void QuickProjectSettings::appTypeChanged( const QString & type )
{
    listForms->setEnabled( type == "GUI" );
    comboMethod->setEnabled( type == "CGI" );
    listQuery->setEnabled( type == "CGI" );
    editBrowser->setEnabled( type == "CGI" );
    if ( type == "GUI" )
	tabConfig->setCurrentPage( 1 );
    else if ( type == "CGI" )
	tabConfig->setCurrentPage( 2 );
    else
	tabConfig->setCurrentPage( 0 );
}

void QuickProjectSettings::addQuery()
{
    QListViewItem *item = new QListViewItem( listQuery );
    item->setRenameEnabled( 0, TRUE );
    item->setRenameEnabled( 1, TRUE );
    qApp->processEvents();
    item->startRename( 0 );
}

void QuickProjectSettings::removeQuery()
{
    QListViewItem *i = listQuery->currentItem();
    delete i;
}

QString QuickProjectSettings::composeQuery()
{
    QListViewItemIterator it( listQuery );
    QString query;
    while ( it.current() ) {
	QString key = it.current()->text( 0 );
	QString value = it.current()->text( 1 );
	QUrl::encode( key );
	QUrl::encode( value );
	if ( !query.isEmpty() )
	    query += "&";
	query += key + "=" + value;
	++it;
    }
    return query;
}

void QuickProjectSettings::setupQueryList( const QString & query )
{
    listQuery->clear();
    QStringList lst = QStringList::split( '&', query );
    for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
	QStringList kv = QStringList::split( '=', *it );
	QListViewItem *item = new QListViewItem( listQuery );
	item->setRenameEnabled( 0, TRUE );
	item->setRenameEnabled( 1, TRUE );
	QString key = kv[0];
	QUrl::decode( key );
	QString value = kv[1];
	QUrl::decode( value );
	item->setText( 0, key );
	item->setText( 1, value );
    }
}

