/****************************************************************************
** $Id: quickstackview.cpp  beta2   edited Jan 22 14:24 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickstackview.h"
#include "quickinterpreter.h"
#include "quickdebugger.h"
#include "quickinterpreterinterfaceimpl.h"
#include "../shared/globaldefs.h"
#include "../qsa/qsproject.h"
#include <qheader.h>
#include <qpainter.h>

QuickStackItem::QuickStackItem( QListView *parent, QListViewItem *after )
    : QListViewItem( parent, after )
{
}

void QuickStackItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align )
{
    QColorGroup g( cg );
    g.setColor( QColorGroup::Base, backgroundColor() );
    g.setColor( QColorGroup::Foreground, Qt::black );
    g.setColor( QColorGroup::Text, Qt::black );
    p->save();

    QListViewItem::paintCell( p, g, column, width, align );
    p->setPen( QPen( cg.dark(), 1 ) );
    if ( column == 0 )
	p->drawLine( 0, 0, 0, height() - 1 );
    p->drawLine( 0, height() - 1, width, height() - 1 );
    p->drawLine( width - 1, 0, width - 1, height() );
    p->restore();
}

QColor QuickStackItem::backgroundColor()
{
    updateBackColor();
    return backColor;
}

void QuickStackItem::updateBackColor()
{
    if ( listView()->firstChild() == this ) {
 	backColor = *backColor1;
	return;
    }

    QListViewItemIterator it( this );
    --it;
    if ( it.current() ) {
	if ( ( ( QuickStackItem*)it.current() )->backColor == *backColor1 )
	    backColor = *backColor2;
	else
	    backColor = *backColor1;
    } else {
	backColor == *backColor1;
    }
}



QuickStackView::QuickStackView( QWidget *parent, const char * )
    : QListView( parent, "quick_debugger_stackview" )
{
    setSorting( -1 );
    init_colors();
    interpreterInterface = new QuickInterpreterInterfaceImpl;
    addColumn( tr( "Line" ) );
    addColumn( tr( "Function" ) );
    setResizeMode( LastColumn );
    setAllColumnsShowFocus( TRUE );
    connect( this, SIGNAL( clicked( QListViewItem * ) ), this, SLOT( functionClicked( QListViewItem * ) ) );
    setHScrollBarMode( AlwaysOff );
    setVScrollBarMode( AlwaysOn );
}

QuickStackView::~QuickStackView()
{
    delete interpreterInterface;
}

void QuickStackView::updateStack()
{
    QValueList<QuickDebuggerStackFrame> lst = QSProject::ideQuickInterpreter()->debuggerEngine()->backtrace();
    clear();
    QuickStackItem *last = 0;
    for ( QValueList<QuickDebuggerStackFrame>::ConstIterator it = lst.begin();
	  it != lst.end(); ++it ) {
	last = new QuickStackItem( this, last );
	last->setText( 1, (*it).function );
	last->setText( 0, QString::number( (*it).line + 1 ) );
	last->setSourceId( (*it).sourceId );
    }
}

void QuickStackView::functionClicked( QListViewItem *i )
{
    if ( !i )
	return;

    QObject *sidObj = QSProject::ideQuickInterpreter()->objectOfSourceId( ( (QuickStackItem*)i )->sourceId() );
    Q_ASSERT( sidObj );
    interpreterInterface->showStackFrame( (QWidget*)sidObj, i->text( 0 ).toInt() - 1 );
}
