/****************************************************************************
** $Id: quickinterpreter.h  beta2   edited Jan 27 12:02 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKINTERPRETER_H
#define QUICKINTERPRETER_H

#ifndef QT_H
#include <qobject.h>
#include <qobjectlist.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qptrvector.h>
#include <qmap.h>
#include "dlldefs.h"
#include <qptrlist.h>
#endif // QT_H

#include "quickclassparser.h"
#include "quickdispatchobject.h"
#include "qsengine.h"
#include <qstypes.h>

class QuickDebugger;
class QuickDispatchObjectFactory;
class QSWrapperClass;
class QSPointerClass;
class QSVariantClass;
class QSMapClass;
class QSPointClass;
class QSSizeClass;
class QSRectClass;
class QSColorClass;
class QSFontClass;
class QSByteArrayClass;
class QSPixmapClass;
class QSApplicationClass;
class QSWrapperShared;

typedef QPtrList<QSWrapperShared> QSWrapperSharedList;

#if defined(Q_TEMPLATEDLL)
// MOC_SKIP_BEGIN
template class QUICKCORE_EXPORT QMap<int, QObject *>;
template class QUICKCORE_EXPORT QMap<QString, QObject*>;
template class QUICKCORE_EXPORT QMap<QString, QString>;
template class QUICKCORE_EXPORT QMap<QString, bool>;
template class QPtrList<QuickDispatchObjectFactory>;
template class QUICKCORE_EXPORT QMap<QString, QPtrList<QuickDispatchObjectFactory> >;
template class QUICKCORE_EXPORT QMap<QObject*, QSWrapperClass*>;
//template class QUICKCORE_EXPORT QStringList;
// MOC_SKIP_END
#endif

extern QMap<QWidget*, QString> *qwf_forms;
extern QString *qwf_language;
extern bool qwf_execute_code;
extern bool qwf_stays_on_top;
extern QObject* qwf_form_object;

class QUICKCORE_EXPORT QuickInterpreter : public QSEngine {
    Q_OBJECT
public:
    QuickInterpreter( bool deb=FALSE );
    virtual ~QuickInterpreter();

    void addTopLevelObject( QObject *o );
    void setTopLevelObjects( QObjectList *l );
    bool hasTopLevelParent( QObject *o );
    QStringList children() const { return kids; }
    QObjectList *topLevelObjects() const { return toplevel; }

    bool checkSyntax( const QString &code );
    QVariant execute( QObject *_this, const QString &code, int &sourceId );
    QVariant call( QObject *ctx, const QString &func, const QSList &args );
    QVariant call( QSObject ctx, const QString &func, const QSList &args );
    QVariant call( QObject *ctx, const QString &func, const QValueList<QVariant> &args );

    void clear();
    void stop();

    bool hadError() const;

    void warn( const QString &msg, int l );

    QuickDebugger *debuggerEngine() const { return debugger; }

    struct Property
    {
	Property() {}
	Property( const QString &n, const QString &t ) : name( n ), type( t ) {}
	QString name;
	QString type;
	bool operator==( const Property &p ) const { return name == p.name && type == p.type; }
    };

    static void enableTimers( bool b );
    static bool timersEnabled();

    void reinit();

    void clearSourceIdMap();
    void addSourceId( int id, QObject *o );
    QObject *objectOfSourceId( int id ) const;
    int sourceIdOfObject( QObject *o ) const;

    bool queryDispatchObjects( QObject *obj, QPtrVector<QObject> &result );
    bool queryDispatchObjects( const QCString &name, void *ptr,
			       QPtrVector<QObject> &result );
    bool construct( const QString &className, const QValueList<QVariant> &args,
		    QPtrVector<QObject> &result );

    // use with caution
    static QuickInterpreter *fromEnv( QSEnv *e );

    static void cleanType( QString &type );
    static void cleanTypeRev( QString &type );

    void emitRunProject() { emit runProject(); }
    void emitStopProject() { emit stopProject(); }
    void emitToggleDebugger( bool enable ) { emit toggleDebugger( enable ); }

    QStringList staticGlobalObjects() const { return staticGlobals; }

    void addDispatchObjectFactory( QuickDispatchObjectFactory *f );
    QPtrList<QuickDispatchObjectFactory> findDispatchObjectFactory( const QString &feature );

    QSWrapperClass *wrapperClass() const { return wrpClass; }
    QSPointerClass *pointerClass() const { return ptrClass; }
    QSVariantClass *variantClass() const { return varClass; }
    QSMapClass *mapClass() const { return mpClass; }
    QSPointClass *pointClass() const { return pntClass; }
    QSSizeClass *sizeClass() const { return sizClass; }
    QSRectClass *rectClass() const { return rctClass; }
    QSColorClass *colorClass() const { return colClass; }
    QSFontClass *fontClass() const { return fntClass; }
    QSByteArrayClass *byteArrayClass() const { return baClass; }
    QSPixmapClass *pixmapClass() const { return pixClass; }
    QSApplicationClass *applicationClass() const { return appClass; }
    QSObject wrap( QObject *o );

    QStringList functionsOf( QSObject &obj, bool includeSignature, bool includeNative = FALSE ) const;
    QStringList classes() const;
    QStringList classesOf( QSObject &obj ) const;
    QStringList variablesOf( QSObject &obj, bool includeStatic = FALSE,
			     bool includeCustom = FALSE ) const;
    QSObject object( const QString &name ) const;
    const QSClass *classOf( const QSObject &obj ) const;

    int uniqueId() const { return id; }
    int userDataId() const { return usrDataId; }

signals:
    void runtimeError();
    void warning( const QString &msg, int line );
    void runProject();
    void stopProject();
    void queryGlobalFunctions( QStringList &funcs );
    void toggleDebugger( bool );

private slots:
    void topLevelDestroyed( QObject *o );

private:
    friend class QuickScriptObject;
    friend class QuickScript;
#if defined(Q_DISABLE_COPY)
    QuickInterpreter( const QuickInterpreter & );
    QuickInterpreter& operator=( const QuickInterpreter & );
#endif

    void init();
    void invalidateWrappers();

    bool shuttingDown;
    QuickDebugger *debugger;
    QObjectList *toplevel;
    QStringList kids;
    QMap<int, QObject *> sourceIdMap;
    QMap<QString, QPtrList<QuickDispatchObjectFactory> > factoryCache;
    QStringList staticGlobals;

    QSWrapperClass *wrpClass;
    QSPointerClass *ptrClass;
    QSVariantClass *varClass;
    QSMapClass *mpClass;
    QSPointClass *pntClass;
    QSSizeClass *sizClass;
    QSRectClass *rctClass;
    QSColorClass *colClass;
    QSFontClass *fntClass;
    QSByteArrayClass *baClass;
    QSPixmapClass *pixClass;
    QSApplicationClass *appClass;
    QObjectList globalObjects;
    QSWrapperSharedList *wrapperShared;
    int id;
    int usrDataId;

};

#endif
