/****************************************************************************
** $Id: quickinterpreterinterfaceimpl.h  beta2   edited Dec 10 13:07 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKINTERPRETERINTERFACEIMPL_H
#define QUICKINTERPRETERINTERFACEIMPL_H

#include <interpreterinterface.h>
#include <qwidget.h>
#include <qmap.h>
#include "dlldefs.h"
#include <qstringlist.h>

class QUICKCORE_EXPORT QuickInterpreterInterfaceImpl : public QObject, public InterpreterInterface
{
    Q_OBJECT

public:
    QuickInterpreterInterfaceImpl();
    ~QuickInterpreterInterfaceImpl();

    QRESULT queryInterface( const QUuid&, QUnknownInterface **iface );
    unsigned long addRef();
    unsigned long release();

    void init() { clear(); }
    bool exec( QObject *obj, const QString &code );
    void setBreakPoints( QObject *obj, const QValueList<uint> &lst );
    void onShowDebugStep( QObject *obj, const char *slot );
    void onShowError( QObject *obj, const char *slot );
    void onShowStackFrame( QObject *obj, const char *slot );
    void onFinish( QObject *obj, const char *slot );
    QString createVariableDeclaration( const QString &var );
    QString createFunctionDeclaration( const QString &name, const QString &body );

    void showDebugStep( QWidget *w, int line );
    void showError( QWidget *w, int line, const QString &message );
    void showStackFrame( QWidget *w, int line );
    void doFinish();

    void clear();

signals:
    void sDebugStep( QObject *, int line );
    void sStackFrame( QObject *, int line );
    void sError( QObject *, int line, const QString &message );
    void finish();

private:
    ulong ref;
    static QObject *debugReceiver;
    static const char *debugSlot;
    static QObject *finishReceiver;
    static const char *finishSlot;
    static QObject *errorReceiver;
    static const char *errorSlot;
    static QObject *stackFrameReceiver;
    static const char *stackFrameSlot;

};

#endif
