/****************************************************************************
** $Id: quicklanguageinterfaceimpl.cpp  beta2   edited Jan 28 14:13 $
**
** Copyright (C) 2001-2003 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quicklanguageinterfaceimpl.h"
#include "quickclassparser.h"
#include "quickcompiler.h"
#include "quickinterpreter.h"
#include "../qsa/qsproject.h"
#include <designerinterface.h>
#include <qobject.h>
#include <qfile.h>
#include <qdom.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qmetaobject.h>
#include <private/qucom_p.h>
#include <qptrvector.h>
#include <qvariant.h>

QuickLanguageInterfaceImpl::QuickLanguageInterfaceImpl()
    : ref( 0 )
{
}

QRESULT QuickLanguageInterfaceImpl::queryInterface( const QUuid &uuid, QUnknownInterface **iface )
{
    *iface = 0;
    if ( uuid == IID_QUnknown )
	*iface = (QUnknownInterface*)this;
    else if ( uuid == IID_Language )
	*iface = (LanguageInterface*)this;
    else
	return QE_NOINTERFACE;

    (*iface)->addRef();
    return QS_OK;
}

unsigned long QuickLanguageInterfaceImpl::addRef()
{
    return ref++;
}

unsigned long QuickLanguageInterfaceImpl::release()
{
    if ( !--ref ) {
	delete this;
	return 0;
    }
    return ref;
}

class NormalizeObject : public QObject
{
public:
    NormalizeObject() : QObject() {}
    static QCString normalizeSignalSlot( const char *signalSlot ) { return QObject::normalizeSignalSlot( signalSlot ); }
};

void QuickLanguageInterfaceImpl::functions( const QString &code, QValueList<Function> *functionMap ) const
{
    QuickClassParser parser;
    parser.parse( code );
    QValueList<QuickClass> classes = parser.classes();
    for ( QValueList<QuickClass>::Iterator it = classes.begin(); it != classes.end(); ++it ) {
	if ( (*it).type == QuickClass::Global ) {
	    *functionMap = (*it).functions;
	    return;
	}
    }
}

void QuickLanguageInterfaceImpl::connections( const QString &code, QValueList<Connection> *conns ) const
{
    QuickClassParser parser;
    parser.parse( code );
    QValueList<QuickClass> classes = parser.classes();
    for ( QValueList<QuickClass>::Iterator it = classes.begin(); it != classes.end(); ++it ) {
	if ( (*it).type == QuickClass::Global ) {
	    *conns = (*it).connections;
	    return;
	}
    }
}

QString QuickLanguageInterfaceImpl::createFunctionStart( const QString &, const QString &func,
							 const QString &returnType,
							 const QString &access )
{
    // ### Not sure about the void. According to Harri, there is a
    // void return type in ECMAScript. This means that we can't use
    // void as default return type in the designer then.
    QString prefix;
    if ( !access.isEmpty() && access != "public" )
	prefix = access + " ";
    if ( returnType.isEmpty() || returnType == "void" )
	return prefix + "function " + func;
    return prefix + "function " + func + " : " + returnType;
}

QStringList QuickLanguageInterfaceImpl::definitions() const
{
    QStringList lst;
    lst << "Class Variables";
    return lst;
}

QStringList QuickLanguageInterfaceImpl::definitionEntries( const QString &definition, QUnknownInterface *designerIface ) const
{
    DesignerInterface *iface = 0;
    designerIface->queryInterface( IID_Designer, (QUnknownInterface**)&iface );
    if ( !iface )
	return QStringList();
    DesignerFormWindow *fw = iface->currentForm();
    if ( !fw )
	return QStringList();
    QStringList lst;
    if ( definition == "Class Variables" ) {
	//lst = fw->variables();
    }
    iface->release();
    return lst;
}

void QuickLanguageInterfaceImpl::setDefinitionEntries( const QString &definition, const QStringList &/*entries*/, QUnknownInterface *designerIface )
{
    DesignerInterface *iface = 0;
    designerIface->queryInterface( IID_Designer, (QUnknownInterface**)&iface );
    if ( !iface )
	return;
    DesignerFormWindow *fw = iface->currentForm();
    if ( !fw )
	return;
    if ( definition == "Class Variables" ) {
	//fw->setVariables( entries );
    }
    iface->release();
}

QString QuickLanguageInterfaceImpl::createArguments( const QString &cpp_signature )
{
    QStringList args = QStringList::split( ',', cpp_signature );
    QString s;
    int i = 0;
    for ( QStringList::ConstIterator it = args.begin(); it != args.end(); ++it ) {
	if ( it != args.begin() )
	    s += ",";
	QString tmp = *it;
	tmp = tmp.simplifyWhiteSpace();
	int pos = tmp.find( ':' );
	if ( pos == -1 ) {
	    if ( tmp.isEmpty() )
		s += "var" + QString::number( ++i );
	    else
		s += tmp;
	} else {
	    QString left = tmp.left( pos ).simplifyWhiteSpace();
	    QString right = tmp.mid( pos + 1 ).simplifyWhiteSpace();
	    if ( !left.isEmpty() ) {
		s += left;
	    } else {
		if ( !right.isEmpty() )
		    s += right + "_var";
		else
		    s += "var" + QString::number( ++i );
	    }
	    if ( !right.isEmpty() )
		s += ":" + right;
	}
    }
    return s;
}

QString QuickLanguageInterfaceImpl::createEmptyFunction()
{
    return "{\n\n}";
}

bool QuickLanguageInterfaceImpl::supports( Support s ) const
{
    if ( s == ReturnType )
	return FALSE;
    if ( s == ConnectionsToCustomSlots )
	return FALSE;
    if ( s == CompressProject )
	return TRUE;
    return FALSE;
}

QStringList QuickLanguageInterfaceImpl::fileFilterList() const
{
    QStringList f;
    f << "Qt Script Source Files (*.qs)";
    return f;

}
QStringList QuickLanguageInterfaceImpl::fileExtensionList() const
{
    QStringList f;
    f << "qs";
    return f;
}

QString QuickLanguageInterfaceImpl::projectKeyForExtension( const QString & ) const
{
    return "DESIGNER_SOURCES";
}

void QuickLanguageInterfaceImpl::sourceProjectKeys( QStringList &keys ) const
{
    keys << "DESIGNER_SOURCES";
}


QString QuickLanguageInterfaceImpl::cleanSignature( const QString &sig )
{
    QString s( sig );
    QString left = s.left( s.find( '(' ) ) + "(";
    s = s.mid( s.find( '(' ) + 1 );
    s = s.left( s.find( ')' ) );
    QStringList lst = QStringList::split( ',', s );
    for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
	if ( it != lst.begin() )
	    left += ",";
	QString p = *it;
	int i = p.find( ':' );
	if ( i == -1 )
	    left += p.simplifyWhiteSpace();
	else
	    left += p.mid( i + 1 ).simplifyWhiteSpace();
    }
    left += ")";
    return left;
}

void QuickLanguageInterfaceImpl::loadFormCode( const QString &, const QString &filename,
					       QValueList<Function> &functions,
					       QStringList &vars,
					       QValueList<Connection> &connections )
{
    QFile f( filename );
    if ( !f.open( IO_ReadOnly ) )
	return;
    QTextStream ts( &f );
    QString code( ts.read() );

    QuickClassParser parser;
    parser.parse( code );
    QValueList<QuickClass> classes = parser.classes();
    for ( QValueList<QuickClass>::Iterator it = classes.begin(); it != classes.end(); ++it ) {
	if ( (*it).type == QuickClass::Global ) {
	    functions = (*it).functions;
	    vars = (*it).variables;
	    connections = (*it).connections;
	    return;
	}
    }
}

bool QuickLanguageInterfaceImpl::canConnect( const QString &signal, const QString &slot )
{
    QString sigArgs = signal;
    QString slotArgs = slot;

    sigArgs = sigArgs.mid( sigArgs.find( '(' ) + 1 );
    sigArgs = sigArgs.left( sigArgs.find( ')' ) );

    slotArgs = slotArgs.mid( slotArgs.find( '(' ) + 1 );
    slotArgs = slotArgs.left( slotArgs.find( ')' ) );

    if ( sigArgs == slotArgs )
	return TRUE;
    if ( sigArgs.contains( ',' ) == slotArgs.contains( ',' ) ) // ### should do type checking
	return TRUE;
    return FALSE;
}

void QuickLanguageInterfaceImpl::preferedExtensions( QMap<QString, QString> &extensionMap ) const
{
    extensionMap.insert( "qs", "Qt Script Source File" );
}

void QuickLanguageInterfaceImpl::compressProject( const QString &projectFile,
						  const QString &compressedFile,
						  bool withWarning )
{
    QuickCompiler compiler;
    if ( compiler.compile( projectFile, compressedFile ) == -1 ) {
	if ( withWarning )
	    QMessageBox::critical( 0, "Could not save project", "Couldn't save the scripting project.\n"
				   "Make sure " + compressedFile + " is writeable" );
    }
}

QString QuickLanguageInterfaceImpl::uncompressProject( const QString &projectFile,
						       const QString &destDir )
{
    QDomDocument doc;
    QString errMsg;
    int errLine;
    QFile f( projectFile );
    if ( !f.open( IO_ReadOnly ) ) {
	QDir d( destDir );
	if ( !d.exists() )
	    d.mkdir( destDir );
	return destDir + "/" + QFileInfo( projectFile ).baseName() + ".pro";
    }

    QTextStream ts( &f );
    QString contents = ts.read();
    f.close();

    QDir d( destDir );
    d.mkdir( destDir );

    if ( !doc.setContent( contents, &errMsg, &errLine ) )
	return destDir + "/" + QFileInfo( projectFile ).baseName() + ".pro";

    QDomElement e = doc.firstChild().toElement().firstChild().toElement();
    QString mainForm, apptype = "GUI", proFile;
    bool madeImgDir = FALSE;
    while ( !e.isNull() ) {
	if ( e.tagName() == "mainform" ) {
	    mainForm = e.firstChild().toText().data();
	} else if ( e.tagName() == "apptype" ) {
	    apptype = e.firstChild().toText().data();
	} else if ( e.tagName() == "projectfile" ) {
	    QByteArray cont;
	    uncompressContents( e.firstChild().toText().data(), e.attribute( "size" ).toInt(), cont );
	    proFile = destDir + "/" + e.attribute( "file" );
	    writeFile( cont, destDir + "/" + e.attribute( "file" ) );
	} else if ( e.tagName() == "uifile" ) {
	    QByteArray cont;
	    uncompressContents( e.firstChild().toText().data(), e.attribute( "size" ).toInt(), cont );
	    writeFile( cont, destDir + "/" + e.attribute( "file" ) );
	} else if ( e.tagName() == "formcode" ) {
	    QByteArray cont;
	    uncompressContents( e.firstChild().toText().data(), e.attribute( "size" ).toInt(), cont );
	    writeFile( cont, destDir + "/" + e.attribute( "file" ) );
	} else if ( e.tagName() == "sourcefile" ) {
	    QByteArray cont;
	    uncompressContents( e.firstChild().toText().data(), e.attribute( "size" ).toInt(), cont );
	    writeFile( cont, destDir + "/" + e.attribute( "file" ) );
	} else if ( e.tagName() == "image" ) {
	    QString s( e.firstChild().toText().data() );
	    QByteArray cont;
	    uncompressContents( s, e.attribute( "size" ).toInt(), cont );
	    if ( madeImgDir ) {
		QDir d( destDir );
		d.mkdir( "image" );
		madeImgDir = TRUE;
	    }
	    writeFile( cont, destDir + "/images/" + e.attribute( "file" ) );
	}
	e = e.nextSibling().toElement();
    }

    return proFile;
}

void QuickLanguageInterfaceImpl::writeFile( const QByteArray &ba, const QString &filename )
{
    QFile f( filename );
    f.open( IO_WriteOnly );
    f.writeBlock( ba );
    f.close();
}

void QuickLanguageInterfaceImpl::uncompressContents( const QString &data, ulong length, QByteArray &baunzip )
{
    const int lengthOffset = 4;
    int baSize = data.length() / 2 + lengthOffset;
    uchar *ba = new uchar[ baSize ];
    for ( int i = lengthOffset; i < baSize; ++i ) {
	char h = data[ 2 * (i-lengthOffset) ].latin1();
	char l = data[ 2 * (i-lengthOffset)  + 1 ].latin1();
	uchar r = 0;
	if ( h <= '9' )
	    r += h - '0';
	else
	    r += h - 'a' + 10;
	r = r << 4;
	if ( l <= '9' )
	    r += l - '0';
	else
	    r += l - 'a' + 10;
	ba[ i ] = r;
    }
    if ( length <  data.length() * 5 )
	length = data.length() * 5;
    // qUncompress() expects the first 4 bytes to be the expected length of
    // the uncompressed data
    ba[0] = ( length & 0xff000000 ) >> 24;
    ba[1] = ( length & 0x00ff0000 ) >> 16;
    ba[2] = ( length & 0x0000ff00 ) >> 8;
    ba[3] = ( length & 0x000000ff );
    if ( data.length() )
	baunzip = qUncompress( ba, baSize );
}

QString QuickLanguageInterfaceImpl::aboutText() const
{
    return "<center><large><b>Qt Scripter</b></large></center>"
	"<center>Version 1.0-alpha</center>"
	"<center>Copyright (c) Trolltech AS 2002</center>";
}

void QuickLanguageInterfaceImpl::addConnection( const QString &sender, const QString &signal,
						const QString &receiver, const QString &slot,
						QString *code )
{
    // ### check if the connection exists
    // ### skip first comment

    code->prepend( "connect( " + sender + ", \"" + signal + "\", " + receiver + ", \"" + slot + "\" );\n" );
}

void QuickLanguageInterfaceImpl::removeConnection( const QString &sender, const QString &signal,
						   const QString &receiver, const QString &slot,
						   QString *code )
{
    QString c = *code;
    int last = 0;
    int firstFunc = code->find( "function" );
    if ( firstFunc == -1 )
	firstFunc = 0xffffff;

    for ( ;; ) {
	int start = c.find( "connect(", last );
	if ( start == -1 || start > firstFunc )
	    return;
	int end = c.find( ';', start );
	if ( end == -1 )
	    return;
	last = start + 1;

	int sender_idx = c.find( sender, start );
	if ( sender_idx == -1 || sender_idx > end )
	    continue;
	int signal_idx = c.find( signal.left( signal.find( '(' ) ) + "(", sender_idx + sender.length() );
	if ( signal_idx == -1 || signal_idx > end )
	    continue;
	int receiver_idx = c.find( receiver, signal_idx + signal.length() );
	if ( receiver_idx == -1 || receiver_idx > end )
	    continue;
	int slot_idx = c.find( slot + "\"", receiver_idx + receiver.length() );
	if ( receiver_idx == -1 || slot_idx > end )
	    continue;

	QString n1 = NormalizeObject::normalizeSignalSlot( signal.ascii() );
	QString sl = c.mid( signal_idx, receiver_idx - signal_idx );
	sl = sl.left( sl.find( '\"' ) );
	QString n2 = NormalizeObject::normalizeSignalSlot( sl.ascii() );

	if ( n1 == n2 ) {
	    code->remove( start, end - start + 1 );
	    int end = code->find( '\n', start );
	    if ( end != -1 )
		code->remove( start, end - start + 1 );
	    return;
	}
    }
}

static void getSignals( QObject* o, QStrList &result, bool super )
{
    QMetaObject* meta = o->metaObject();
    int nsignals = meta->numSignals( super );
    for ( int i = 0; i < nsignals; i++ ) {
	const QMetaData* md = meta->signal( i, super );
	QString s = QString::fromLatin1( md->name );
	s = s.left( s.find( '(' ) );
	s += "(";
	const QUMethod *m = md->method;
	for ( int j = 0; j < m->count; ++j ) {
	    QUParameter p = m->parameters[j];
	    if ( j == 0 )
		s += " ";
	    QString type = p.type->desc();
	    if ( type == "ptr" )
		type = (const char*)p.typeExtra;
	    else if ( type == "varptr" )
		type = QVariant::typeToName( (QVariant::Type)*(char*)p.typeExtra );
	    QuickInterpreter::cleanType( type );
	    s += QString( p.name ) + " : " + type;
	    if ( j < m->count - 1 )
		s += ", ";
	    else
		s += " ";
	}
	s += ")";
	if ( result.find( s ) == -1 )
	    result.append( s );
    }
}

QStrList QuickLanguageInterfaceImpl::signalNames( QObject *obj ) const
{
    QPtrVector<QObject> objects;
    QSProject::ideQuickInterpreter()->queryDispatchObjects( obj, objects );
    QStrList res;
    for ( int i = objects.size() - 1; i >= 0; --i )
        getSignals( objects[ i ], res, i == 0 );
    return res;
}
