/****************************************************************************
** $Id: quicklanguageinterfaceimpl.h  beta2   edited Dec 10 13:07 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#ifndef QUICKLANGUAGEINTERFACEIMPL_H
#define QUICKLANGUAGEINTERFACEIMPL_H

#include <languageinterface.h>
#include "dlldefs.h"

class QUICKCORE_EXPORT QuickLanguageInterfaceImpl : public LanguageInterface
{
public:
    QuickLanguageInterfaceImpl();

    QRESULT queryInterface( const QUuid&, QUnknownInterface **iface );
    unsigned long addRef();
    unsigned long release();

    void functions( const QString &code, QValueList<Function> *funcs ) const;
    void connections( const QString &code, QValueList<Connection> *conns ) const;
    QString createFunctionStart( const QString &className, const QString &func,
				 const QString &returnType, const QString &access );
    QStringList definitions() const;
    QStringList definitionEntries( const QString &definition, QUnknownInterface *designerIface ) const;
    void setDefinitionEntries( const QString &definition, const QStringList &entries, QUnknownInterface *designerIface );
    QString createArguments( const QString &cpp_signature );
    QString createEmptyFunction();
    bool supports( Support s ) const;
    QStringList fileFilterList() const;
    QStringList fileExtensionList() const;
    void preferedExtensions( QMap<QString, QString> &extensionMap ) const;
    QString projectKeyForExtension( const QString &extension ) const;
    void sourceProjectKeys( QStringList &keys ) const;
    QString cleanSignature( const QString &sig );
    void loadFormCode( const QString &form, const QString &filename,
		       QValueList<Function> &functions,
		       QStringList &vars,
		       QValueList<Connection> &connections );
    QString formCodeExtension() const { return ".qs"; }
    bool canConnect( const QString &signal, const QString &slot );

    void compressProject( const QString &projectFile, const QString &compressedFile, bool withWarning );
    QString uncompressProject( const QString &projectFile, const QString &destDir );
    QString aboutText() const;

    void addConnection( const QString &sender, const QString &signal,
			const QString &receiver, const QString &slot,
			QString *code );
    void removeConnection( const QString &sender, const QString &signal,
			   const QString &receiver, const QString &slot,
			   QString *code );
    QStrList signalNames( QObject *obj ) const;

private:
    void writeFile( const QByteArray &ba, const QString &filename );
    void uncompressContents( const QString &data, ulong length, QByteArray &baunzip );

    ulong ref;

};

#endif
